@@ -21,13 +21,24 @@ from apio.profile import Profile
 from apio.managers.downloader import FileDownloader
 from apio.managers.unpacker import FileUnpacker
 
+# -- Packages marked as obsoletes
+# -- The value is the replacement package
+OBSOLETE_PKG = {"system": "oss-cad-suite"}
+
 
 class Installer:
-    """DOC: TODO"""
+    """Installer. Class with methods for installing and managing
+    packages"""
 
     def __init__(self, package, platform="", force=False, checkversion=True):
+        """Class initialization. Parameters:
+        * package:  Package name to manage/install. It can have a prefix with
+                    the version. Ex. "system@1.1.2"
+        """
 
-        # Parse version
+        # Parse version. The following attributes are used:
+        #  * Installer.package: Package name (without version)
+        #  * Installer.version: Package version (or None)
         if "@" in package:
             split = package.split("@")
             self.package = split[0]
@@ -36,47 +47,75 @@ class Installer:
             self.package = package
             self.version = None
 
+        # -- Attribute Installer.force_install
+        # -- Force installation or not
         self.force_install = force
+
+        # -- Installer.package_dir: path were the packages are stored
+        # -- Ex. /home/obijuan/.apio/packages
         self.packages_dir = ""
 
+        # -- Get all the resources for the given platform
+        # -- Some resources depend on the platform (like the packages)
+        # -- but some others don't (like the boards)
         self.resources = Resources(platform)
 
         self.profile = Profile()
 
+        # -- Folder name were the packages are stored
         dirname = "packages"
 
+        # -- If the package is known...
+        # --(It is defined in the resources/packages.json file)
         if self.package in self.resources.packages:
 
+            # -- Store the package dir
             self.packages_dir = util.safe_join(util.get_home_dir(), dirname)
 
-            # Get data
+            # Get the data of the given package
             data = self.resources.packages.get(self.package)
+
+            # Get the information about the valid versions
             distribution = self.resources.distribution
 
+            # Get the spectec package version
             self.spec_version = distribution.get("packages").get(self.package)
+
+            # Get the package name (from resources/package.json file)
             self.package_name = data.get("release").get("package_name")
+
+            # Get the extension given to the toolchain. Tipically tar.gz
             self.extension = data.get("release").get("extension")
+
+            # Get the current platform
             platform = platform or self._get_platform()
 
+            # Check if the version is ok (It is only done if the
+            # checkversion flag has been activated)
             if checkversion:
 
-                # Check version
+                # Check version. The filename is read from the
+                # repostiroy
                 valid_version = self._get_valid_version(
                     data.get("repository").get("name"),
                     data.get("repository").get("organization"),
                     data.get("release").get("tag_name"),
                 )
 
-                # Valid version
+                # No valid version found
                 if not valid_version:
                     # Error
                     click.secho("Error: no valid version found", fg="red")
                     sys.exit(1)
 
+                # Store the valid version
                 self.version = valid_version
 
+                # Get the plaform_os name
                 # e.g., [linux_x86_64, linux]
                 platform_os = platform.split("_")[0]
+
+                # Build the URLs for downloading the package
                 self.download_urls = [
                     {
                         "url": self.get_download_url(data, platform),
@@ -87,14 +126,17 @@ class Installer:
                         "platform": platform_os,
                     },
                 ]
-
+        # -- The package is kwnown but the version is not correct
         else:
+
             if self.package in self.profile.packages and checkversion is False:
                 self.packages_dir = util.safe_join(
                     util.get_home_dir(), dirname
                 )
                 self.package_name = "toolchain-" + package
 
+        # -- If the Installer.package_dir is empty, is because the package
+        # -- was not known. Abort!
         if self.packages_dir == "":
             click.secho(
                 "Error: no such package '{}'".format(self.package), fg="red"
@@ -125,29 +167,46 @@ class Installer:
         return download_url
 
     def install(self):
-        """DOC: TODO"""
+        """Install the current package in the set in the Installer Object"""
 
-        click.echo(
-            "Installing %s package:" % click.style(self.package, fg="cyan")
-        )
+        # -- Warning if the package has been marked as obsolete
+        if self.package in self.resources.obsolete_pkgs:
+            click.secho(
+                f"Warning: Package {self.package} is obsolete. "
+                f"Will be removed in the future. "
+                f"Use {OBSOLETE_PKG[self.package]} instead\n",
+                fg="yellow",
+            )
+
+        click.secho(f"Installing {self.package} package:", fg="cyan")
+
+        # -- Create the apio package folder, if it does not exit
         if not isdir(self.packages_dir):
             makedirs(self.packages_dir)
         assert isdir(self.packages_dir)
+
+        # -- The first step is downloading the package
+        # -- This variable stores the path to tha packages
         dlpath = None
+
         try:
             # Try full platform
             platform_download_url = self.download_urls[0].get("url")
             dlpath = self._download(platform_download_url)
+
+        # -- There is no write access to the package folder
         except IOError as exc:
             click.secho(
                 "Warning: permission denied in packages directory", fg="yellow"
             )
             click.secho(str(exc), fg="red")
+
+        # --
         except Exception:
             # Try os name
             dlpath = self._install_os_package(platform_download_url)
 
-        # Install downloaded package
+        # -- Second step: Install downloaded package
         self._install_package(dlpath)
 
         # Rename unpacked dir to package dir
@@ -247,9 +306,13 @@ class Installer:
         return tarball
 
     def _get_valid_version(self, rel_name, organization, tag_name):
+        """Get the latest valid version. It can be the latest vesion available
+        in github or a specific version"""
 
-        # Download latest releases list
-        releases = api_request("{}/releases".format(rel_name), organization)
+        # Get the data from the Github api
+        # Ex of the object obtained in this URL:
+        # https://api.github.com/repos/FPGAwars/tools-oss-cad-suite/releases
+        releases = api_request(f"{rel_name}/releases", organization)
 
         if self.version:
 
