@@ -1,4 +1,4 @@
-"""DOC: TODO"""
+"""Implementation of the Apio system commands"""
 # -*- coding: utf-8 -*-
 # -- This file is part of the Apio project
 # -- (C) 2016-2019 FPGAwars
@@ -7,6 +7,8 @@
 
 import re
 import platform
+from pathlib import Path
+from os.path import isfile
 import click
 
 from apio import util
@@ -15,22 +17,38 @@ from apio.resources import Resources
 
 
 class System:  # pragma: no cover
-    """DOC: TODO"""
+    """System class. Managing and execution of the system commands"""
 
     def __init__(self):
+
+        # -- Read the profile from the file
         profile = Profile()
+
+        # -- Read the resources from the corresponding files
         resources = Resources()
 
+        # -- This command is called system
         self.name = "system"
+
+        # -- Package name: apio package were all the system commands
+        # -- are located
+        # -- From apio > 0.7 the system tools are located inside the
+        # -- oss-cad-suite
+        self.package_name = "oss-cad-suite"
+
+        # -- Get the installed package versions
         self.version = util.get_package_version(self.name, profile)
+
+        # -- Get the spected versions
         self.spec_version = util.get_package_spec_version(self.name, resources)
 
+        # -- Windows: Executables should end with .exe
         self.ext = ""
         if platform.system() == "Windows":
             self.ext = ".exe"
 
     def lsusb(self):
-        """DOC: TODO"""
+        """Run the lsusb system command"""
 
         returncode = 1
         result = self._run_command("lsusb")
@@ -99,22 +117,79 @@ class System:  # pragma: no cover
 
     def _run_command(self, command, silent=False):
         result = {}
-        system_base_dir = util.get_package_dir("tools-system")
-        system_bin_dir = util.safe_join(system_base_dir, "bin")
 
+        print(f"(DEBUG) Run Command: {command}")
+
+        # From apio >= 0.7.0, the system tools are locate in the
+        # oss-cad-suite package instead of the system package
+        # So first let's try to execute them from there
+
+        # -- Get the package base dir
+        system_base_dir = util.get_package_dir("tools-oss-cad-suite")
+        print(f"(DEBUG) System_base_dir: {system_base_dir}")
+
+        # -- Get the folder were the binary file is locateds
+        system_bin_dir = Path(system_base_dir) / "bin"
+        print(f"(DEBUG) System bin dir: {system_bin_dir}")
+
+        # -- Get the executable filename
+        executable_file = system_bin_dir / (command + self.ext)
+        print(f"(DEBUG) Executable file: {executable_file}")
+
+        # -- Set the stdout and stderr for executing the command
         on_stdout = None if silent else self._on_stdout
         on_stderr = self._on_stderr
 
-        if util.check_package(
-            self.name, self.version, self.spec_version, system_bin_dir
-        ):
+        # -- Check if the executable exists
+        if isfile(executable_file):
+
+            # -- Execute the command!
             result = util.exec_command(
-                util.safe_join(system_bin_dir, command + self.ext),
+                executable_file,
                 stdout=util.AsyncPipe(on_stdout),
                 stderr=util.AsyncPipe(on_stderr),
             )
 
-        return result
+            # -- Return the result of the execution
+            return result
+
+        # -- The command does not exist in the tools-oss-cad-suite package
+        # -- Try with the tool-system package (old-package,
+        # -- (for compatibility reasons)
+
+        # -- Get the package base dir
+        system_base_dir = util.get_package_dir("tools-system")
+        print(f"System_base_dir: {system_base_dir}")
+
+        # -- Get the folder were the binary file is locateds
+        system_bin_dir = Path(system_base_dir) / "bin"
+        print(f"System bin dir: {system_bin_dir}")
+
+        # -- Get the executable filename
+        executable_file = system_bin_dir / (command + self.ext)
+        print(f"Executable file: {executable_file}")
+
+        # -- Check if the executable exists
+        if isfile(executable_file):
+
+            # -- Execute the command!
+            result = util.exec_command(
+                executable_file,
+                stdout=util.AsyncPipe(on_stdout),
+                stderr=util.AsyncPipe(on_stderr),
+            )
+
+            # -- Return the result of the execution
+            return result
+
+        # -- The command was not in the oss-cad-suit package
+        # -- The command was not in the old system package
+        # -- Show the error message and a hint on how to install the package
+        print("ERROR!!!!!!")
+        util.show_package_path_error(self.package_name)
+        util.show_package_install_instructions(self.package_name)
+
+        return None
 
     @staticmethod
     def _on_stdout(line):
