@@ -17,9 +17,9 @@ import locale
 import platform
 import subprocess
 from threading import Thread
-from os.path import expanduser, isdir, isfile, join, dirname, exists, normpath
+from os.path import isdir, isfile, join, dirname, exists, normpath
+from pathlib import Path
 
-import jwt
 import click
 import semantic_version
 from serial.tools.list_ports import comports
@@ -27,6 +27,44 @@ import requests
 
 from apio import LOAD_CONFIG_DATA
 
+# ----------------------------------------
+# -- Constants
+# ----------------------------------------
+
+# -- Packages names
+# -- If you need to create new packages, you should
+# -- define first the constants here
+# --
+OSS_CAD_SUITE = "oss-cad-suite"
+SYSTEM = "system"
+SCONS = "scons"
+GTKWAVE = "gtkwave"
+IVERILOG = "iverilog"
+VERILATOR = "verilator"
+YOSYS = "yosys"
+ICE40 = "ice40"
+ECP5 = "ecp5"
+FUJPROG = "fujprog"
+ICESPROG = "icesprog"
+DFU = "dfu"
+
+# -- Name of the subfolder to store de executable files
+BIN = "bin"
+
+# -- Folder names. They are built from the
+# -- packages names
+OSS_CAD_SUITE_FOLDER = f"tools-{OSS_CAD_SUITE}"
+SYSTEM_FOLDER = f"tools-{SYSTEM}"
+SCONS_FOLDER = f"tool-{SCONS}"
+GTKWAVE_FOLDER = f"tool-{GTKWAVE}"
+IVERILOG_FOLDER = f"toolchain-{IVERILOG}"
+VERILATOR_FOLDER = f"toolchain-{VERILATOR}"
+YOSYS_FOLDER = f"toolchain-{YOSYS}"
+ICE40_FOLDER = f"toolchain-{ICE40}"
+ECP5_FOLDER = f"toolchain-{ECP5}"
+FUJPROG_FOLDER = f"toolchain-{FUJPROG}"
+ICESPROG_FOLDER = f"toolchain-{ICESPROG}"
+DFU_FOLDER = f"toolchain-{DFU}"
 
 requests.packages.urllib3.disable_warnings()
 
@@ -144,7 +182,7 @@ def safe_join(*paths):
 
 def _get_config_data():
     """Return the configuration data located in the /etc/apio.json file
-       Only for Debian Distribution. It will return None otherwise """
+    Only for Debian Distribution. It will return None otherwise"""
 
     # Default value
     _config_data = None
@@ -168,11 +206,15 @@ config_data = _get_config_data()
 
 
 def _get_projconf_option_dir(name, default=None):
-    """Return project the project option with the given name
-       These options are place either on environment variables or
-       into the /etc/apio.json file in the case of debian distributions
+    """Return the project option with the given name
+    These options are place either on environment variables or
+    into the /etc/apio.json file in the case of debian distributions
 
-       All the APIO environment variables have the prefix "APIO_"
+    All the APIO environment variables have the prefix "APIO_"
+
+    Project options:
+
+    * home_dir : APIO home directory
     """
 
     # -- Get the full name of the environment variable
@@ -192,7 +234,7 @@ def _get_projconf_option_dir(name, default=None):
             _env_value = _env_value[1:-1]
 
         # -- Debug: Print the environment variable (without quotes)
-        print(f"{_env_name}: {_env_value}")
+        # print(f"DEBUG: {_env_name}: {_env_value}")
 
         return _env_value
 
@@ -203,87 +245,79 @@ def _get_projconf_option_dir(name, default=None):
         # -- Return the value of the option
         return config_data.get(_env_name)
 
+    # -- Return the default home_dir
     return default
 
 
 def get_home_dir():
-    """Get the APIO Home dir. This is the apio folder where the profle is located
-    and the packages installes. The APIO Home dir can be set in the APIO_HOME_DIR
-    environment varible or in the /etc/apio.json file (in Debian). If not set, the  
-    user_HOME/.apio folder is used by default:
+    """Get the APIO Home dir. This is the apio folder where the profle is
+    located and the packages installed. The APIO Home dir can be set in the
+    APIO_HOME_DIR environment varible or in the /etc/apio.json file (in
+    Debian). If not set, the user_HOME/.apio folder is used by default:
     Ej. Linux:  /home/obijuan/.apio
     If the folders does not exist, they are created
     It returns a list with all the folders
     """
 
-    # -- Get the APIO HOME DIR
-    home_dir = _get_projconf_option_dir("home_dir", "~/.apio")
+    # -- Get the APIO_HOME_DIR env variable
+    # -- It returns None if it was not defined
+    apio_home_dir_env = _get_projconf_option_dir("home_dir")
 
-    # -- Expand the ~ symbol with the full home user folder
-    home_dir = re.sub(r"\~", expanduser("~").replace("\\", "/"), home_dir)
+    # -- Get the home dir. It is what the APIO_HOME_DIR env variable
+    # -- says, or the default folder if None
+    if apio_home_dir_env:
+        home_dir = Path(apio_home_dir_env)
+    else:
+        home_dir = Path.home() / ".apio"
 
-    # -- Parse the path and convert it into its folder names
-    paths = home_dir.split(os.pathsep)
+    # -- Create the folders if they do not exist
+    try:
+        home_dir.mkdir(parents=True, exist_ok=True)
+    except PermissionError:
+        click.secho(f"Error: no usable home directory {home_dir}", fg="red")
+        sys.exit(1)
 
-    # -- Check if the folders already exist
-    path = _check_writable(paths)
-
-    # -- No folders yet
-    if not path:
-
-        # -- Create the folders
-        path = _create_path(paths)
-
-        # -- Still no luck (maybe no permission to write)
-        if not path:
-            click.secho("Error: no usable home directory " + path, fg="red")
-            sys.exit(1)
-
-    # Return the home_dir as a list of all its folders   
-    print(path);      
-    return path
-
-
-def _check_writable(paths):
-    ret = ""
-    for path in paths:
-        if isdir(path):
-            if os.access(path, os.W_OK):
-                # Path is writable
-                ret = path
-                break
-            click.secho("Warning: can't write in path " + path, fg="yellow")
-    return ret
-
-
-def _create_path(paths):
-    ret = ""
-    for path in paths:
-        if not isdir(path):
-            try:
-                os.makedirs(path)
-                ret = path
-                break
-            except OSError as ioex:
-                if ioex.errno == 13:
-                    click.secho("Warning: can't create " + path, fg="yellow")
-    return ret
+    # Return the home_dir as a string
+    # In the future it should return the path object
+    return str(home_dir)
 
 
 def get_package_dir(pkg_name):
-    """DOC: TODO"""
+    """Return the APIO package dir of a given package
+    Packages are installed in the following folder:
+    * Default: $APIO_HOME_DIR/packages
+    * $APIO_PKG_DIR/packages: if the APIO_PKG_DIR env variable is set
+    * Return a String
+    """
 
-    home_dir = _get_projconf_option_dir("pkg_dir", "")
-    if not home_dir:
-        home_dir = _get_projconf_option_dir("home_dir", "~/.apio")
-    home_dir = re.sub(r"\~", expanduser("~").replace("\\", "/"), home_dir)
+    # -- Get the apio home dir:
+    apio_home_dir = get_home_dir()
 
-    paths = home_dir.split(os.pathsep)
-    for path in paths:
-        package_dir = safe_join(path, "packages", pkg_name)
-        if isdir(package_dir):
-            return package_dir
+    # -- Get the APIO_PKG_DIR env variable
+    # -- It returns None if it was not defined
+    apio_pkg_dir_env = _get_projconf_option_dir("pkg_dir")
 
+    # -- Get the pkg base dir. It is what the APIO_PKG_DIR env variable
+    # -- says, or the default folder if None
+    if apio_pkg_dir_env:
+        pkg_home_dir = Path(apio_pkg_dir_env)
+
+    # -- Default value
+    else:
+        pkg_home_dir = Path(apio_home_dir)
+
+    # -- Create the package folder
+    package_dir = pkg_home_dir / "packages" / pkg_name
+
+    # -- Return the folder if it exists
+    if package_dir.exists():
+        return str(package_dir)
+
+    # -- Show an error message (for debugging)
+    # click.secho(f"Folder does not exists: {package_dir}", fg="red")
+    # sys.exit(1)
+
+    # -- Return a null string if the folder does not exist
     return ""
 
 
@@ -294,44 +328,41 @@ def get_project_dir():
 
 
 def call(cmd):
-    """DOC: TODO"""
+    """Execute the given command from the installed apio packages"""
 
+    # -- Set the PATH environment variable for finding the
+    # -- executables on the apio package folders first
     setup_environment()
-    return subprocess.call(cmd, shell=True)
+
+    # -- Execute the command from the shell
+    result = subprocess.call(cmd, shell=True)
+
+    # -- Command not found
+    if result == 127:
+        message = f"ERROR. Comand not found!: {cmd}"
+        click.secho(message, fg="red")
+
+    return result
 
 
 def setup_environment():
-    """DOC: TODO"""
+    """Set the environment variables and the system PATH"""
 
-    base_dir = {
-        "scons": get_package_dir("tool-scons"),
-        "system": get_package_dir("tools-system"),
-        "yosys": get_package_dir("toolchain-yosys"),
-        "ice40": get_package_dir("toolchain-ice40"),
-        "ecp5": get_package_dir("toolchain-ecp5"),
-        "iverilog": get_package_dir("toolchain-iverilog"),
-        "verilator": get_package_dir("toolchain-verilator"),
-        "gtkwave": get_package_dir("tool-gtkwave"),
-        "fujprog": get_package_dir("toolchain-fujprog"),
-        "icesprog": get_package_dir("toolchain-icesprog"),
-        "dfu": get_package_dir("toolchain-dfu"),
-        "openfpgaloader": get_package_dir("toolchain-ecp5"),
-    }
+    # --- Get the table with the paths of all the apio packages
+    base_dir = get_base_dir()
 
-    bin_dir = {
-        "scons": safe_join(base_dir.get("scons"), "script"),
-        "system": safe_join(base_dir.get("system"), "bin"),
-        "yosys": safe_join(base_dir.get("yosys"), "bin"),
-        "ice40": safe_join(base_dir.get("ice40"), "bin"),
-        "ecp5": safe_join(base_dir.get("ecp5"), "bin"),
-        "iverilog": safe_join(base_dir.get("iverilog"), "bin"),
-        "verilator": safe_join(base_dir.get("verilator"), "bin"),
-        "gtkwave": safe_join(base_dir.get("gtkwave"), "bin"),
-        "fujprog": safe_join(base_dir.get("fujprog"), "bin"),
-        "icesprog": safe_join(base_dir.get("icesprog"), "bin"),
-        "dfu": safe_join(base_dir.get("dfu"), "bin"),
-        "openfpgaloader": safe_join(base_dir.get("openfpgaloader"), "bin"),
-    }
+    # --- Get the table with tha paths of all the executables
+    # --- of the apio packages
+    bin_dir = get_bin_dir_table(base_dir)
+
+    # --- Set the system env. variables
+    set_env_variables(base_dir, bin_dir)
+
+    return bin_dir
+
+
+def set_env_variables(base_dir, bin_dir):
+    """Set the environment variables"""
 
     # Give the priority to the python packages installed with apio
     os.environ["PATH"] = os.pathsep.join([get_bin_dir(), os.environ["PATH"]])
@@ -339,74 +370,59 @@ def setup_environment():
     # Give the priority to the packages installed by apio
     os.environ["PATH"] = os.pathsep.join(
         [
-            bin_dir.get("yosys"),
-            bin_dir.get("ice40"),
-            bin_dir.get("ecp5"),
-            bin_dir.get("iverilog"),
-            bin_dir.get("verilator"),
-            bin_dir.get("fujprog"),
-            bin_dir.get("icesprog"),
-            bin_dir.get("dfu"),
-            bin_dir.get("openfpgaloader"),
+            bin_dir.get(OSS_CAD_SUITE),
+            bin_dir.get(YOSYS),
+            bin_dir.get(ICE40),
+            bin_dir.get(ECP5),
+            bin_dir.get(IVERILOG),
+            bin_dir.get(VERILATOR),
+            bin_dir.get(FUJPROG),
+            bin_dir.get(ICESPROG),
+            bin_dir.get(DFU),
+            bin_dir.get(SYSTEM),
             os.environ["PATH"],
         ]
     )
 
+    # -- Add the gtkwave to the path, only in windows
     if platform.system() == "Windows":
         os.environ["PATH"] = os.pathsep.join(
-            [bin_dir.get("gtkwave"), os.environ["PATH"]]
+            [bin_dir.get(GTKWAVE), os.environ["PATH"]]
         )
 
     # Add environment variables
     if not config_data:  # /etc/apio.json file does not exist
-        os.environ["IVL"] = safe_join(base_dir.get("iverilog"), "lib", "ivl")
-    os.environ["ICEBOX"] = safe_join(base_dir.get("ice40"), "share", "icebox")
-    os.environ["TRELLIS"] = safe_join(base_dir.get("ecp5"), "share", "trellis")
-    os.environ["YOSYS_LIB"] = safe_join(
-        base_dir.get("yosys"), "share", "yosys"
-    )
+        os.environ["IVL"] = safe_join(base_dir.get(IVERILOG), "lib", "ivl")
 
-    return bin_dir
+    os.environ["ICEBOX"] = str(Path(base_dir.get(ICE40)) / "share" / "icebox")
+
+    os.environ["TRELLIS"] = str(Path(base_dir.get(ECP5)) / "share" / "trellis")
+
+    os.environ["YOSYS_LIB"] = str(
+        Path(base_dir.get(YOSYS)) / "share" / "yosys"
+    )
 
 
 def resolve_packages(packages, installed_packages, spec_packages):
-    """DOC: TODO"""
+    """Check the given packages.
+    * Check that they are installed
+    * Check that the versions are ok"""
 
-    base_dir = {
-        "scons": get_package_dir("tool-scons"),
-        "system": get_package_dir("tools-system"),
-        "yosys": get_package_dir("toolchain-yosys"),
-        "ice40": get_package_dir("toolchain-ice40"),
-        "ecp5": get_package_dir("toolchain-ecp5"),
-        "iverilog": get_package_dir("toolchain-iverilog"),
-        "verilator": get_package_dir("toolchain-verilator"),
-        "gtkwave": get_package_dir("tool-gtkwave"),
-        "fujprog": get_package_dir("toolchain-fujprog"),
-        "icesprog": get_package_dir("toolchain-icesprog"),
-        "dfu": get_package_dir("toolchain-dfu"),
-        "openfpgaloader": get_package_dir("toolchain-ecp5"),
-    }
+    # --- Get the table with the paths of all the apio packages
+    base_dir = get_base_dir()
 
-    bin_dir = {
-        "scons": safe_join(base_dir.get("scons"), "script"),
-        "system": safe_join(base_dir.get("system"), "bin"),
-        "yosys": safe_join(base_dir.get("yosys"), "bin"),
-        "ice40": safe_join(base_dir.get("ice40"), "bin"),
-        "ecp5": safe_join(base_dir.get("ecp5"), "bin"),
-        "iverilog": safe_join(base_dir.get("iverilog"), "bin"),
-        "verilator": safe_join(base_dir.get("verilator"), "bin"),
-        "gtkwave": safe_join(base_dir.get("gtkwave"), "bin"),
-        "fujprog": safe_join(base_dir.get("fujprog"), "bin"),
-        "icesprog": safe_join(base_dir.get("icesprog"), "bin"),
-        "dfu": safe_join(base_dir.get("dfu"), "bin"),
-        "openfpgaloader": safe_join(base_dir.get("openfpgaloader"), "bin"),
-    }
+    # --- Get the table with the paths of all the executables
+    # --- of the apio packages
+    bin_dir = get_bin_dir_table(base_dir)
 
     # -- Check packages
     check = True
     for package in packages:
+        
         version = installed_packages.get(package, {}).get("version", "")
+
         spec_version = spec_packages.get(package, "")
+
         check &= check_package(
             package, version, spec_version, bin_dir.get(package)
         )
@@ -414,58 +430,73 @@ def resolve_packages(packages, installed_packages, spec_packages):
     # -- Load packages
     if check:
 
-        # Give the priority to the python packages installed with apio
-        os.environ["PATH"] = os.pathsep.join(
-            [get_bin_dir(), os.environ["PATH"]]
-        )
-
-        # Give the priority to the packages installed by apio
-        os.environ["PATH"] = os.pathsep.join(
-            [
-                bin_dir.get("yosys"),
-                bin_dir.get("ice40"),
-                bin_dir.get("ecp5"),
-                bin_dir.get("iverilog"),
-                bin_dir.get("verilator"),
-                bin_dir.get("fujprog"),
-                bin_dir.get("icesprog"),
-                bin_dir.get("dfu"),
-                bin_dir.get("openfpgaloader"),
-                os.environ["PATH"],
-            ]
-        )
-
-        if platform.system() == "Windows":
-            os.environ["PATH"] = os.pathsep.join(
-                [bin_dir.get("gtkwave"), os.environ["PATH"]]
-            )
-
-        # Add environment variables
-        if not config_data:  # /etc/apio.json file does not exist
-            os.environ["IVL"] = safe_join(
-                base_dir.get("iverilog"), "lib", "ivl"
-            )
-        os.environ["ICEBOX"] = safe_join(
-            base_dir.get("ice40"), "share", "icebox"
-        )
-        os.environ["TRELLIS"] = safe_join(
-            base_dir.get("ecp5"), "share", "trellis"
-        )
-        os.environ["YOSYS_LIB"] = safe_join(
-            base_dir.get("yosys"), "share", "yosys"
-        )
+        # --- Set the system env. variables
+        set_env_variables(base_dir, bin_dir)
 
         global scons_command
         scons_command = [
             normpath(sys.executable),
-            safe_join(bin_dir["scons"], "scons"),
+            safe_join(bin_dir[SCONS], "scons"),
         ]
 
     return check
 
 
+def get_base_dir():
+    """Return the table with the local paths of the apio packages
+    installed on the system"""
+
+    # -- Create the table:
+    # --  Package Name  :  Folder (string)
+    base_dir = {
+        OSS_CAD_SUITE: get_package_dir(OSS_CAD_SUITE_FOLDER),
+        SCONS: get_package_dir(SCONS_FOLDER),
+        YOSYS: get_package_dir(YOSYS_FOLDER),
+        ICE40: get_package_dir(ICE40_FOLDER),
+        ECP5: get_package_dir(ECP5_FOLDER),
+        IVERILOG: get_package_dir(IVERILOG_FOLDER),
+        VERILATOR: get_package_dir(VERILATOR_FOLDER),
+        GTKWAVE: get_package_dir(GTKWAVE_FOLDER),
+        FUJPROG: get_package_dir(FUJPROG_FOLDER),
+        ICESPROG: get_package_dir(ICESPROG_FOLDER),
+        DFU: get_package_dir(DFU_FOLDER),
+        # -- Obsolete packages
+        SYSTEM: get_package_dir(SYSTEM_FOLDER),
+    }
+
+    return base_dir
+
+
+def get_bin_dir_table(base_dir):
+    """Return a table with the package name and the folder were
+    the executable files are stored
+    * Input: Table with the package base_dir
+    """
+
+    bin_dir = {
+        OSS_CAD_SUITE: str(Path(base_dir.get(OSS_CAD_SUITE)) / BIN),
+        SCONS: str(Path(base_dir.get(SCONS)) / "script"),
+        YOSYS: str(Path(base_dir.get(YOSYS)) / BIN),
+        ICE40: str(Path(base_dir.get(ICE40)) / BIN),
+        ECP5: str(Path(base_dir.get(ECP5)) / BIN),
+        IVERILOG: str(Path(base_dir.get(IVERILOG)) / BIN),
+        VERILATOR: str(Path(base_dir.get(VERILATOR)) / BIN),
+        GTKWAVE: str(Path(base_dir.get(GTKWAVE)) / BIN),
+        FUJPROG: str(Path(base_dir.get(FUJPROG)) / BIN),
+        ICESPROG: str(Path(base_dir.get(ICESPROG)) / BIN),
+        DFU: str(Path(base_dir.get(DFU)) / BIN),
+        # -- Obsolete package
+        SYSTEM: str(Path(base_dir.get(SYSTEM)) / BIN),
+    }
+
+    return bin_dir
+
+
 def check_package(name, version, spec_version, path):
-    """DOC: TODO"""
+    """Check if the given package is installed
+    * name: Package name
+    * path: path where the binary files of the package are stored
+    """
 
     # Apio package 'gtkwave' only exists for Windows.
     # Linux and MacOS user must install the native GTKWave.
@@ -512,7 +543,7 @@ def show_package_version_warning(name, version, spec_version):
 def show_package_path_error(name):
     """DOC: TODO"""
 
-    message = "Error: package '{}' is not installed".format(name)
+    message = f"Error: package '{name}' is not installed"
     click.secho(message, fg="red")
 
 
@@ -521,13 +552,11 @@ def show_package_install_instructions(name):
 
     if config_data and _check_apt_get():  # /etc/apio.json file exists
         click.secho(
-            "Please run:\n" "   apt-get install apio-{}".format(name),
+            f"Please run:\n   apt-get install apio-{name}",
             fg="yellow",
         )
     else:
-        click.secho(
-            "Please run:\n" "   apio install {}".format(name), fg="yellow"
-        )
+        click.secho(f"Please run:\n   apio install {name}", fg="yellow")
 
 
 def _check_apt_get():
@@ -575,7 +604,7 @@ def exec_command(*args, **kwargs):  # pragma: no cover
     default = dict(
         stdout=subprocess.PIPE,
         stderr=subprocess.PIPE,
-        shell=platform.system() == "Windows",
+        shell=False,
     )
     default.update(kwargs)
     kwargs = default
@@ -691,12 +720,6 @@ def command(function):
     return decorate
 
 
-def decode(text):
-    """DOC: TODO"""
-
-    return jwt.decode(text, "secret", algorithm="HS256")
-
-
 def get_serial_ports():
     """DOC: TODO"""
 
@@ -743,6 +766,6 @@ def get_bin_dir():
 
 
 def get_python_version():
-    """DOC: TODO"""
+    """Return a string with the python version"""
 
-    return "{0}.{1}".format(sys.version_info[0], sys.version_info[1])
+    return f"{sys.version_info[0]}.{sys.version_info[1]}"
